#!/usr/bin/ruby
#
# Sensu Handler: mailer
#
# This handler formats alerts as mails and sends them off to a pre-defined recipient.
#
# Copyright 2019 Seji64
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

# Note: The default matrix config is fetched from the predefined json config file which is "matrix.json" or any other
#       file defined using the "json_config" command line option.

require 'sensu-handler'
require 'matrix_sdk'
require 'erb'

class Mailer < Sensu::Handler
    option :json_config,
           description: 'Name of the JSON Configuration block in Sensu (default: matrix)',
           short: '-j JsonConfig',
           long: '--json_config JsonConfigKey',
           required: false,
           default: 'matrix'

    def fetch_setting(setting_key)
        config_key = config[:json_config]  
        value = @event[setting_key]
        value ||= settings[config_key][setting_key] if config_key
    end

    def json_config
        @json_config ||= config[:json_config]
    end

    def action_to_string
        actions = {
            'create' => 'Created',
            'resolve' => 'Resolved',
            'flapping' => 'Flapping'
        }
        actions[@event['action']]
    end

    def action_symbol
        icons = {
          'create' => "\xF0\x9F\x98\xB1",
          'resolve' => "\xF0\x9F\x98\x8D",
          'flapping' => "\xF0\x9F\x90\x9D"
        }
        icons[@event['action']]
    end

    def check_status
        @event['check']['status']
    end

    def occurrences
        @event['occurrences']
    end
    def status_symbol
        symbol = {
            0 => '✔',
            1 => '⚠',
            2 => '❌',
            3 => '❓'
        }
        symbol[check_status.to_i]
    end

    def status_to_string
        status = {
            0 => 'OK',
            1 => 'Warning',
            2 => 'Critical',
            3 => 'Unknown'
        }
        status[check_status.to_i]
    end

    def short_name
        client_name + '/' + check_name
    end

    def client_name
        @event['client']['name']
    end
    
    def check_name
        @event['check']['name']
    end

    def output
        (@event['check']['output']).to_s.gsub(/(\s-p|\s-P|\s--password)(\s*\S+)/, '\1 <password omitted>')
    end

    def command
        (@event['check']['command']).to_s.gsub(/(\s-p|\s-P|\s--password)(\s*\S+)/, '\1 <password omitted>')
    end

    def room_id
        fetch_setting 'matrix_room_id'
    end
    
    def username
        fetch_setting 'matrix_user'
    end
    
    def token
        fetch_setting 'matrix_access_token'
    end

    def password
        fetch_setting 'matrix_password'
    end

    def server
        fetch_setting 'matrix_server'
    end

    def timeout
        fetch_setting 'timeout' || 10
    end

    def default_message
        [
          '<b>Alert <%= action_to_string %></b> <%= action_symbol %>',
          '<b>Host:</b> <%= client_name %>',
          '<b>Check:</b> <%= check_name %>',
          '<b>Status:</b> <%= status_to_string %> <%= status_symbol %>',
          '<b>Occurrences:</b> <%= occurrences %>',
          '<b>Output:<b> <pre><%= output %></pre>'
        ].join("<br>")
    end

    def build_message
        template_file = fetch_setting 'message_template_file'
        if !template_file.nil?
            template = File.read(template_file)
        else
            template = fetch_setting 'message_template'
            template ||= default_message
        end
        message = ERB.new(template).result(binding)

        message
    end

    def handle

        ENV['SSL_CERT_FILE'] = '/etc/pki/tls/certs/ca-bundle.crt'

        if settings[json_config].nil?
            p "invalid config: #{options[:json_config]} you need to pass a key and not a file"
            exit 3 # unknown
        end

        begin
            Timeout.timeout timeout do
                client = MatrixSdk::Client.new "https://#{server}"
                if !token.nil?
                    client.api.access_token = token
                    client.sync
                else
                    client.login username, password
                end
                room = client.find_room room_id
                room.send_html build_message
                # Logout if we logged in before with username and password
                if token.nil?
                    client.logout
                end
            end  
            puts 'matrix -- sent alert for #{short_name} to room with id #{room_id}'
        rescue Timeout::Error
            puts 'matrix -- timed out while attempting to ' + @event['action'] + ' an incident -- #{short_name}'
        rescue StandardError => e
            puts "Matrix handler error '#{e.inspect}' while attempting to report an incident: #{short_name} #{action_to_string}"    
        end
    end
end